package dev.aouahib.medialist

import android.content.Context
import android.database.Cursor
import android.os.Looper
import android.provider.MediaStore
import androidx.annotation.NonNull
import androidx.loader.content.CursorLoader
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers.IO
import kotlinx.coroutines.Dispatchers.Main
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/** MedialistPlugin */
class MedialistPlugin : FlutterPlugin, MethodCallHandler {
    
    companion object {
        const val kMethodListMedia = "listMedia"
        const val kMethodCountMedia = "countMedia"
        const val kTypes = "types"
        const val kThumbSize = "thumbSize"
        const val kPage = "page"
        const val kMediaPerPage = "mediaPerPage"
        const val kMediaPath = "mediaPath"
        const val kThumbnailPath = "thumbnailPath"
        const val kType = "type"
        const val kDuration = "duration"
        private const val kTypeImage = "image"
        private const val kTypeVideo = "video"
        private const val kThumbSizeMini = "mini"
        private const val kThumbSizeMicro = "micro"
        private const val kSourceExternal = "external"

        val platformTypes: Map<String, Int> = mapOf(
                kTypeImage to MediaStore.Files.FileColumns.MEDIA_TYPE_IMAGE,
                kTypeVideo to MediaStore.Files.FileColumns.MEDIA_TYPE_VIDEO
        )
        
        val supportedThumbnailSizes = arrayListOf("mini", "micro")
        
        val platformThumbnailSizes = mapOf(
                kThumbSizeMini to mapOf(
                        platformTypes[kTypeImage] to MediaStore.Images.Thumbnails.MINI_KIND,
                        platformTypes[kTypeVideo] to MediaStore.Video.Thumbnails.MINI_KIND
                ),
                kThumbSizeMicro to mapOf(
                        platformTypes[kTypeImage] to MediaStore.Images.Thumbnails.MICRO_KIND,
                        platformTypes[kTypeVideo] to MediaStore.Video.Thumbnails.MICRO_KIND
                )
        )
        
        private lateinit var channel: MethodChannel
        lateinit var context: Context
    }


    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "medialist")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            kMethodListMedia -> listMedia(call, result)
            kMethodCountMedia  -> countMedia(call, result)
            else -> result.notImplemented()
        }
    }

    private fun listMedia(call: MethodCall, result: Result) {
        val args = MedialistUtils.extractListingArgs(call, result) ?: return
        CoroutineScope(IO).launch {
            listMediaCoroutine(result, args)
        }
    }

    private fun countMedia(call: MethodCall, result: Result) {
        val args = MedialistUtils.extractCountingArgs(call, result) ?: return


            CoroutineScope(IO).launch {
                countMediaCoroutine(result, args)
            }
    }

    private suspend fun listMediaCoroutine(result: Result, args: ListingArgs) {
        // from where to get media
        val queryUri = MediaStore.Files.getContentUri(kSourceExternal)
        // what type of media to get
        val mediaType = MediaStore.Files.FileColumns.MEDIA_TYPE
        var selection = "$mediaType=${args.types[0]}"
        for (i in 1 until args.types.size)
            selection += " OR $mediaType=${args.types[i]}"
        // what data to get on those media
        val projection = arrayOf(
                MediaStore.Files.FileColumns._ID,
                MediaStore.Files.FileColumns.DATA,
                MediaStore.Files.FileColumns.DATE_ADDED,
                MediaStore.Files.FileColumns.MEDIA_TYPE,
                MediaStore.Files.FileColumns.MIME_TYPE,
                MediaStore.Files.FileColumns.TITLE,
                MediaStore.Video.VideoColumns.DURATION
        )
        // how to sort the media
        val sortOrder = MediaStore.Files.FileColumns.DATE_ADDED + " DESC LIMIT " + args.mediaPerPage + " OFFSET " + args.page * args.mediaPerPage

        if (Looper.myLooper() == null) Looper.prepare()
        val cursorLoader = CursorLoader(
                context,
                queryUri,
                projection,
                selection,
                null,
                sortOrder
        )

        val cursor: Cursor? = cursorLoader.loadInBackground()
        if (cursor == null) {
            sendError(
                    MedialistErrors.cursorError,
                    MedialistErrors.cursorLoadInBackgroundReturnedNull,
                    result
            )
            return
        }

        val dataColumn: Int
        val typeColumn: Int
        val idColumn: Int
        val durationColumn: Int
        try {
            dataColumn = cursor.getColumnIndexOrThrow(MediaStore.Files.FileColumns.DATA)
            typeColumn = cursor.getColumnIndexOrThrow(MediaStore.Files.FileColumns.MEDIA_TYPE)
            idColumn = cursor.getColumnIndexOrThrow(MediaStore.Files.FileColumns._ID)
            durationColumn = cursor.getColumnIndex(MediaStore.Video.VideoColumns.DURATION)
        } catch (e: IllegalArgumentException) {
            sendError(
                    MedialistErrors.cursorError,
                    MedialistErrors.cursorGetIndexOrThrowThrewException, result
            )
            return
        }

        val mediaList = mutableListOf<Map<String, Any>>()
        while (cursor.moveToNext()) {
            val data = cursor.getString(dataColumn)
            val type = cursor.getInt(typeColumn)
            val id = cursor.getLong(idColumn)
            val duration = cursor.getLong(durationColumn)

            lateinit var thumbnail : String
            try {
                thumbnail = MedialistUtils.getThumbnail(
                        mediaId = id,
                        mediaType = type,
                        thumbSize = platformThumbnailSizes[args.thumbSize]?.get(type)
                )
            }  catch (e: Exception) {
                sendError(
                        MedialistErrors.cursorError,
                        MedialistErrors.cursorGetIndexOrThrowThrewException,
                        result
                )
                return

            }
            if (thumbnail == "") {
                withContext(Main) {
                    
                }
                sendError(
                        MedialistErrors.thumbnailError,
                        MedialistErrors.thumbnailCouldNotCreate,
                        result
                )
                return
            }
            mediaList += mapOf(
                    kMediaPath to data,
                    kThumbnailPath to thumbnail,
                    kType to type,
                    kDuration to duration
            )
        }
        sendSuccess(mediaList, result)
        cursor.close()
    }

    private suspend fun countMediaCoroutine(result: Result, args: CountingArgs) {
        // from where to get media
        val queryUri = MediaStore.Files.getContentUri(kSourceExternal)
        // what type of media to get
        val mediaType = MediaStore.Files.FileColumns.MEDIA_TYPE
        var selection = "$mediaType=${args.types[0]}"
        for (i in 1 until args.types.size)
            selection += " OR $mediaType=${args.types[i]}"
        // what data to get on those media
        val projection = arrayOf("count(" + MediaStore.Files.FileColumns._ID + ") as count")
        if (Looper.myLooper() == null) Looper.prepare()
        val cursorLoader = CursorLoader(
                context,
                queryUri,
                projection,
                selection,
                null,
                null
        )
        val cursor : Cursor? = cursorLoader.loadInBackground()
        if (cursor == null) {
            sendError(
                    MedialistErrors.cursorError,
                    MedialistErrors.cursorLoadInBackgroundReturnedNull,
                    result
            )
            return
        }
        cursor.moveToFirst()
        val count : Int = cursor.getInt(0)
        sendSuccess(count, result)
        cursor.close()
    }

    private suspend fun sendSuccess(mediaList: Any, result: Result) {
        withContext(Main)  {
            result.success(mediaList)
        }
    }
    
    private suspend fun sendError(code: String, message: String, result: Result) {
       withContext(Main)  {
           result.error(code, message, "")
       }
    }
}

class ListingArgs(
        val types: List<Int>,
        val thumbSize: String,
        val page: Int,
        val mediaPerPage: Int
)

class CountingArgs(
        val types: List<Int>
)




