package dev.aouahib.medialist

import android.graphics.Bitmap
import android.provider.MediaStore
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.io.File
import java.io.FileOutputStream

class MedialistUtils {
    companion object {
        fun extractListingArgs(
                call: MethodCall,
                result: MethodChannel.Result
        ): ListingArgs? {
            val types = call.argument<ArrayList<String>>(MedialistPlugin.kTypes)
            val thumbSize = call.argument<String>(MedialistPlugin.kThumbSize)
            val page : Int?
            var mediaPerPage = call.argument<Int>(MedialistPlugin.kMediaPerPage)
            if (mediaPerPage != null && mediaPerPage == 0) {
                mediaPerPage = Int.MAX_VALUE
                page = 0
            } else {
                page = call.argument<Int>(MedialistPlugin.kPage)
            }

            if (types == null || types.isEmpty() || thumbSize == null
                    || page == null || mediaPerPage == null
            ) {
                result.error(
                        MedialistErrors.missingArgumentError,
                        MedialistErrors.missingArgumentMessage,
                        ""
                )
                return null
            }

            if (page < 0 || mediaPerPage <= 0) {
                result.error(
                        MedialistErrors.invalidArgumentError,
                        MedialistErrors.invalidArgumentMessage,
                        ""
                )
                return null
            }

            val typesAsIntegers = extractTypes(types, result) ?: return null


            if (thumbSizeInvalid(thumbSize)) {
                result.error(
                        MedialistErrors.unsupportedThumbSizeError,
                        "${MedialistErrors.unsupportedThumbSizeMessage}: $thumbSize",
                        ""
                )
                return null
            }

            return ListingArgs(
                    types = typesAsIntegers,
                    thumbSize = thumbSize,
                    page = page,
                    mediaPerPage = mediaPerPage
            )
        }

        fun extractCountingArgs(
                call: MethodCall,
                result: MethodChannel.Result
        ): CountingArgs? {
            val types = call.argument<ArrayList<String>>(MedialistPlugin.kTypes)
            if (types == null || types.isEmpty()) {
                result.error(
                        MedialistErrors.missingArgumentError,
                        MedialistErrors.missingArgumentMessage,
                        ""
                )
                return null
            }

            val typesAsIntegers = extractTypes(types, result) ?: return null
            
            return CountingArgs(typesAsIntegers)
        }

        private fun extractTypes(types : List<String>, result: MethodChannel.Result)
                : List<Int>? {
            val typesAsIntegers = mutableListOf<Int>()
            for (type in types) {
                val typeAsInt = MedialistPlugin.platformTypes[type]
                if (typeAsInt == null) {
                    result.error(
                            MedialistErrors.unsupportedTypeError,
                            MedialistErrors.unsupportedTypeMessage,
                            ""
                    )
                    return null
                }
                typesAsIntegers += typeAsInt
            }
            return typesAsIntegers
        }

        private fun thumbSizeInvalid(thumbSize: String): Boolean {
            return !MedialistPlugin.supportedThumbnailSizes.contains(thumbSize)
        }

        fun getThumbnail(
                mediaId: Long,
                mediaType: Int,
                thumbSize: Int?
        ): String {
            // this should never happen 
            if (thumbSize == null) return ""

            val context = MedialistPlugin.context
            val thumbName = "${mediaId}_$thumbSize.jpeg"
            val file = File(context.cacheDir, thumbName)

            // if the thumbnail already exists, return its path
            if (file.exists()) return file.path

            // else, create it first and return its path
            val contentResolver = context.contentResolver
            lateinit var thumbnail: Bitmap
            try {
                thumbnail = (
                        when (mediaType) {
                            MediaStore.Files.FileColumns.MEDIA_TYPE_VIDEO ->
                                MediaStore.Video.Thumbnails.getThumbnail(
                                        contentResolver,
                                        mediaId,
                                        thumbSize,
                                        null
                                )
                            MediaStore.Files.FileColumns.MEDIA_TYPE_IMAGE ->
                                MediaStore.Images.Thumbnails.getThumbnail(
                                        contentResolver,
                                        mediaId,
                                        thumbSize,
                                        null
                                )

                            else -> null
                        }
                        ) ?: return "" // this should never happen
            } catch (e: Exception) {
            }

            try {
                val out = FileOutputStream(file)
                thumbnail.compress(Bitmap.CompressFormat.JPEG, 100, out)
                out.flush()
                out.close()
            } catch (e: Exception) {
                print(e)
                return ""
            }
            return file.path
        }
    }
}
