import 'dart:async';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:medialist/medialist.dart';

void main() {
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Builder(builder: (context) {
        return Scaffold(
          appBar: AppBar(
            title: Text('Medialist'),
          ),
          body: Center(
            child: RaisedButton(
              onPressed: () {
                Navigator.push(context,
                    MaterialPageRoute(builder: (_) => GalleryScreen()));
              },
              child: Text('View Gallery'),
            ),
          ),
        );
      }),
    );
  }
}

class GalleryScreen extends StatefulWidget {
  @override
  _GalleryScreenState createState() => _GalleryScreenState();
}

class _GalleryScreenState extends State<GalleryScreen> {
  // how many media items to fetch per call
  static const _mediaPerPage = 10;
  // what media type to include
  static const _types = [MediaType.image, MediaType.video];
  // what thumbnail size to get
  static const _thumbSize = MediaThumbSize.mini;
  // Medialist object
  final Medialist medialist = Medialist();
  // Total media items count
  int mediaCount;
  // The actual media items
  List<MediaItem> medias;
  // current page
  int page;
  // should stop fetching?
  bool stop;
  // we currently fetching?
  bool fetching;

  @override
  void initState() {
    super.initState();
    mediaCount = 0;
    page = 0;
    medias = [];
    stop = false;
    fetching = false;
    _firstFetch();
  }

  Future<void> _firstFetch() async {
    // we get the total media items count in the storage, so we can use it
    // to know exactly how many placeholders we'll display
    medialist
        .countMedia(types: _types)
        .then(
          (int result) => setState(
            () => mediaCount = result,
          ),
        )
        .catchError((e) => print(e));

    // we fetch the first page of media items
    medialist
        .listMedia(
          thumbSize: _thumbSize,
          page: page,
          mediaPerPage: _mediaPerPage,
          types: _types,
        )
        .then((result) => setState(() {
              medias = result;
              stop = result.length < _mediaPerPage;
            }))
        .catchError((e) => print(e));
  }

  Future<void> _fetchMore() async {
    // If we're in the middle of fetching, fetch no more
    // if we already hit the final page, fetch no more
    if (fetching || stop) return;
    // if not, set the fetching state to true
    fetching = true;
    try {
      // we get the next page of media items
      final moreMedia = await medialist.listMedia(
        thumbSize: _thumbSize,
        // incrementing the page
        page: ++page,
        mediaPerPage: _mediaPerPage,
        types: _types,
      );
      // we know we hit the last page if the number of fetched items
      // is less than expected
      stop = moreMedia.length < _mediaPerPage;
      // we're done, set the fetching state back to false
      fetching = false;
      // add the fetched items to the already existing list
      setState(() {
        medias.addAll(moreMedia);
      });
    } on PlatformException catch (e) {
      print(e);
    }
  }

  @override
  Widget build(BuildContext context) {
    final size = MediaQuery.of(context).size;
    return Scaffold(
      appBar: AppBar(
        title: Text('Gallery'),
      ),
      body: GridView.builder(
        gridDelegate: SliverGridDelegateWithMaxCrossAxisExtent(
          maxCrossAxisExtent: size.width / 3,
          childAspectRatio: 1,
        ),
        itemCount: mediaCount,
        itemBuilder: (_, index) {
          MediaItem item;
          // if the index is in range, we get the media item and display it
          // if it is out of range, we fetch more, and display a placeholder
          // meanwhile
          if (index < medias.length) {
            item = medias[index];
          } else {
            _fetchMore();
          }
          return item == null
              ? Container(
                  height: 100,
                  width: 100,
                  color: Colors.amber,
                )
              : Container(
                  height: 100,
                  width: 100,
                  decoration: BoxDecoration(
                    image: DecorationImage(
                      fit: BoxFit.cover,
                      image: FileImage(
                        File.fromUri(
                          Uri.file(item.thumbnailPath),
                        ),
                      ),
                    ),
                  ),
                  child: Material(
                    color: Colors.transparent,
                    child: Stack(
                      fit: StackFit.expand,
                      children: [
                        InkWell(
                          onTap: () {
                            if (item.type == MediaType.image) {
                              Navigator.of(context).push(
                                MaterialPageRoute(
                                  builder: (_) =>
                                      PicturePage(picPath: item.mediaPath),
                                ),
                              );
                            }
                          },
                        ),
                        if (item.type == MediaType.video)
                          Positioned(
                            bottom: 0,
                            left: 0,
                            child: Row(
                              children: [
                                Icon(
                                  Icons.play_arrow,
                                  color: Colors.white,
                                ),
                                Text(
                                  item.displayableDuration(),
                                  style: TextStyle(color: Colors.white),
                                ),
                              ],
                            ),
                          ),
                      ],
                    ),
                  ),
                );
        },
      ),
    );
  }
}

class PicturePage extends StatelessWidget {
  const PicturePage({
    Key key,
    @required this.picPath,
  }) : super(key: key);

  final String picPath;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.black,
      body: Center(child: Image.file(File.fromUri(Uri.file(picPath)))),
    );
  }
}
