import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:permission_handler/permission_handler.dart';

class Medialist {
  static const MethodChannel _channel = MethodChannel('medialist');
  static const _kMethodListMedia = 'listMedia';
  static const _kMethodCountMedia = 'countMedia';
  static const _kParamTypes = 'types';
  static const _kParamThumbSize = 'thumbSize';
  static const _kParamPage = 'page';
  static const _kParamMediaPerPage = 'mediaPerPage';
  static const _kParamMediaPath = 'mediaPath';
  static const _kParamThumbnailPath = 'thumbnailPath';
  static const _kParamType = 'type';
  static const _kParamDuration = 'duration';
  static const _kParamTypeImage = 'image';
  static const _kParamTypeVideo = 'video';
  static const _kParamThumbSizeMini = 'mini';
  static const _kParamThumbSizeMicro = 'micro';

  /// Returns a list of images and/or videos from the internal storage.
  ///
  /// Use [types] to get only images, only videos, or both mixed.
  /// Thumbnails are created and cached, use [thumbSize] to determine the size
  /// of the created thumbnail, see [MediaThumbSize] for size details.
  /// [page] and [mediaPerPage] are used for pagination. Use the default values
  /// to get everything in a single fetch.
  Future<List<MediaItem>> listMedia({
    List<MediaType> types = MediaType.values,
    MediaThumbSize thumbSize = MediaThumbSize.mini,
    int page = 0,
    int mediaPerPage = 0,
  }) async {
    final storagePermission = await Permission.storage.request();
    if (!storagePermission.isGranted) return [];
    final _mediaList = await _channel.invokeListMethod<Map>(
      _kMethodListMedia,
      {
        _kParamTypes:
            types.map((t) => _Serializers.mediaTypeToString(t)).toList(),
        _kParamThumbSize: _Serializers.mediaThumbSizeToString(thumbSize),
        _kParamPage: page,
        _kParamMediaPerPage: mediaPerPage,
      },
    );
    return _mediaList
        .map(
            (e) => _Serializers.mediaItemFromJson(Map<String, dynamic>.from(e)))
        .toList();
  }

  /// Returns the number of images and/or videos in the internal storage.
  ///
  /// Use [types] to determine what media type to include in the count.
  Future<int> countMedia({
    List<MediaType> types = MediaType.values,
  }) async {
    final storagePermission = await Permission.storage.request();
    if (!storagePermission.isGranted) return 0;
    final count = await _channel.invokeMethod<int>(_kMethodCountMedia, {
      _kParamTypes:
          types.map((t) => _Serializers.mediaTypeToString(t)).toList(),
    });
    return count;
  }
}

/// A class that represents an image/video.
class MediaItem {
  MediaItem({
    @required this.mediaPath,
    @required this.thumbnailPath,
    @required this.type,
    @required this.duration,
  });

  /// The absolute path to this item
  final String mediaPath;

  /// The absolute path to item's thumbnail
  final String thumbnailPath;

  /// The type of this item
  final MediaType type;

  /// If the type is [MediaItem.video], this hold the duration in milliseconds
  /// If not, it's always 0
  final int duration;

  /// Returns a string representation of this item's duration (e.g: 13:37)
  String displayableDuration() {
    int seconds = duration ~/ 1000;
    final minutes = seconds ~/ 60;
    seconds = seconds % 60;
    return '$minutes:${seconds.toString().padLeft(2, '0')}';
  }
}

/// Represents the media types
enum MediaType {
  image,
  video,
}

/// Represents the thumbnail sizes
enum MediaThumbSize {
  /// 512 x 384
  mini,

  /// 96 x 96
  micro,
}

class _Serializers {
  static MediaItem mediaItemFromJson(Map<String, dynamic> json) {
    return MediaItem(
      mediaPath: json[Medialist._kParamMediaPath] as String,
      thumbnailPath: json[Medialist._kParamThumbnailPath] as String,
      type: mediaTypeFromInt(json[Medialist._kParamType] as int),
      duration: json[Medialist._kParamDuration] as int,
    );
  }

  static MediaType mediaTypeFromInt(int type) {
    switch (type) {
      case 1:
        return MediaType.image;
      default:
        return MediaType.video;
    }
  }

  static String mediaTypeToString(MediaType type) {
    switch (type) {
      case MediaType.image:
        return Medialist._kParamTypeImage;
      default:
        return Medialist._kParamTypeVideo;
    }
  }

  static String mediaThumbSizeToString(MediaThumbSize thumbSize) {
    switch (thumbSize) {
      case MediaThumbSize.mini:
        return Medialist._kParamThumbSizeMini;
      default:
        return Medialist._kParamThumbSizeMicro;
    }
  }
}
